<?php
/**
 * @package Joomla
 * @subpackage Abook
 * @copyright (C) 2010 Ugolotti Federica
 * @license GNU/GPL, see LICENSE.php
 * EventList is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License 2
 * as published by the Free Software Foundation.

 * Abook is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.

 * You should have received a copy of the GNU General Public License
 * along with Abook; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */
defined( '_JEXEC' ) or die( 'Restricted access' );

jimport( 'joomla.application.component.modellist' );

class AbookModelBooks extends JModelList
{
        public function __construct($config = array())
        {
                if (empty($config['filter_fields'])) {
                        $config['filter_fields'] = array(
                                'id', 'a.id',
                                'title', 'a.title',
                                'alias', 'a.alias',
                                'checked_out', 'a.checked_out',
                                'checked_out_time', 'a.checked_out_time',
                                'catid', 'a.catid', 'category_title',
				'ideditor', 'a.ideditor',
                                'published', 'a.published',
                                'access', 'a.access', 'access_level',
                                'dateinsert', 'a.dateinsert',
                                'user_id', 'a.user_id',
                                'ordering', 'a.ordering',
				'catalogo', 'a.catalogo',
                                'language', 'a.language',
                                'hits', 'a.hits',
				'year', 'a.year',
				'idlocation', 'a.idlocation',
				'idlibrary', 'a.idlibrary',
                        );
                }

                parent::__construct($config);
        }
        protected function populateState($ordering = null, $direction = null)
        {
		$app = JFactory::getApplication();
		if ($layout = JRequest::getVar('layout')) {
                        $this->context .= '.'.$layout;
                }
		
		$search = $this->getUserStateFromRequest($this->context.'.filter.search', 'filter_search');
                $this->setState('filter.search', $search);

		$published = $this->getUserStateFromRequest($this->context.'.filter.published', 'filter_published', '');
                $this->setState('filter.published', $published);

		$categoryId = $this->getUserStateFromRequest($this->context.'.filter.category_id', 'filter_category_id');
                $this->setState('filter.category_id', $categoryId);

		$language = $this->getUserStateFromRequest($this->context.'.filter.language', 'filter_language', '');
                $this->setState('filter.language', $language);

		parent::populateState('a.title', 'asc');
        }

	protected function getStoreId($id = '')
        {
                // Compile the store id.
                $id.= ':' . $this->getState('filter.search');
                $id.= ':' . $this->getState('filter.category_id');
		$id.= ':'.$this->getState('filter.published');
		$id.= ':' . $this->getState('filter.language');

                return parent::getStoreId($id);
        }

	protected function getListQuery()
        {
		$user   = JFactory::getUser();
		$db = $this->getDbo();
                $query = $db->getQuery(true);
		$query->select(
                        $this->getState(
                                'list.select',
                                'a.*')
                );
		$query->from('#__abbook AS a');

		// Join over the language
                $query->select('l.title AS language_title');
                $query->join('LEFT', '`#__languages` AS l ON l.lang_code = a.language');
		
		$query->select('c.title AS category');
                $query->join('LEFT', '#__abcategories AS c ON c.id = a.catid');

		$query->select('ee.name AS editor, ee.id AS editorid');
                $query->join('LEFT', '#__abeditor AS ee ON ee.id = a.ideditor');

                $query->join('LEFT', '#__abbookauth AS aa ON aa.idbook = a.id');

		$query->select('GROUP_CONCAT(bb.name) AS author');
                $query->join('LEFT', '#__abauthor AS bb ON bb.id=aa.idauth');

                $query->join('LEFT', '#__users AS u ON u.id = a.checked_out');

		$query->select('lc.name AS location');
                $query->join('LEFT', '#__ablocations AS lc ON lc.id = a.idlocation');

		$query->select('lb.name AS library');
                $query->join('LEFT', '#__ablibrary AS lb ON lb.id = a.idlibrary');

		$query->select('g.title AS access_level');
                $query->join('LEFT', '#__viewlevels AS g ON g.id = a.access');

		// Filter by published state
                $published = $this->getState('filter.published');
                if (is_numeric($published)) {
                        $query->where('a.published = ' . (int) $published);
                } else if ($published === '') {
                        $query->where('(a.published = 0 OR a.published = 1)');
                }

		// Filter by a single or group of categories.
                $categoryId = $this->getState('filter.category_id');
                if (is_numeric($categoryId)) {
                        $query->where('a.catid = '.(int) $categoryId);
                } else if (is_array($categoryId)) {
                        JArrayHelper::toInteger($categoryId);
                        $categoryId = implode(',', $categoryId);
                        $query->where('a.catid IN ('.$categoryId.')');
                }

		// Filter by search in title.
		$search = $this->getState('filter.search');
                if (!empty($search)) {
			$search = $db->Quote('%'.$db->getEscaped($search, true).'%');
                        $query->where('(a.title LIKE '.$search.' OR a.alias LIKE '.$search.')');
                }

		// Filter on the language.
                if ($language = $this->getState('filter.language')) {
                        $query->where('a.language = '.$db->quote($language));
                }
		
		// Implement View Level Access
                if (!$user->authorise('core.admin'))
                {
                    $groups     = implode(',', $user->getAuthorisedViewLevels());
                        $query->where('a.access IN ('.$groups.')');
                }

		// Add the list ordering clause.
                $orderCol       = $this->state->get('list.ordering');
                $orderDirn      = $this->state->get('list.direction');
                if ($orderCol == 'a.ordering' || $orderCol == 'category') {
                        $orderCol = 'category '.$orderDirn.', a.ordering';
                }
                $query->order($db->getEscaped($orderCol.' '.$orderDirn));

		$query->group('a.id');
                return $query;
        }

	public function getItems()
        {
                $items  = parent::getItems();
                $app    = JFactory::getApplication();
                if ($app->isSite()) {
                        $user   = JFactory::getUser();
                        $groups = $user->getAuthorisedViewLevels();

                        for ($x = 0, $count = count($items); $x < $count; $x++) {
                                //Check the access level. Remove articles the user shouldn't see
                                if (!in_array($items[$x]->access, $groups)) {
                                        unset($items[$x]);
                                }
                        }
                }
                return $items;
        }
}
