<?php
/**
 * @package Joomla
 * @subpackage Abook
 * @copyright (C) 2010 Ugolotti Federica
 * @license GNU/GPL, see LICENSE.php
 * EventList is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License 2
 * as published by the Free Software Foundation.

 * Abook is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.

 * You should have received a copy of the GNU General Public License
 * along with Abook; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */
defined( '_JEXEC' ) or die( 'Restricted access' );

jimport( 'joomla.application.component.modellist' );

class AbookModelCategories extends JModelList
{
	public function __construct($config = array())
        {
                if (empty($config['filter_fields'])) {
                        $config['filter_fields'] = array(
                                'id', 'c.id',
                                'title', 'c.title',
                                'alias', 'c.alias',
                                'published', 'c.published',
                                'access', 'c.access', 'access_level',
                                'language', 'c.language',
                                'checked_out', 'c.checked_out',
                                'checked_out_time', 'c.checked_out_time',
                                'created_time', 'c.created_time',
                                'created_user_id', 'c.created_user_id',
                                'lft', 'c.lft',
                                'rgt', 'c.rgt',
                                'level', 'c.level',
                                'path', 'c.path',
                        );
                }

                parent::__construct($config);
        }

	protected function populateState($ordering = null, $direction = null)
        {
                // Initialise variables.
                $app            = JFactory::getApplication();
		if ($layout = JRequest::getVar('layout', 'default')) {
                        $this->context .= '.'.$layout;
                }

		$extension="com_abook";
		$this->setState('filter.extension', $extension);

                $search = $app->getUserStateFromRequest($this->context.'.search', 'filter_search');
                $this->setState('filter.search', $search);

		$level = $this->getUserStateFromRequest($this->context.'.filter.level', 'filter_level', 0, 'int');
                $this->setState('filter.level', $level);

		$access = $this->getUserStateFromRequest($this->context.'.filter.access', 'filter_access', 0, 'int');
                $this->setState('filter.access', $access);

                $published = $app->getUserStateFromRequest($this->context.'.published', 'filter_published', '');
                $this->setState('filter.published', $published);

		$language = $this->getUserStateFromRequest($this->context.'.filter.language', 'filter_language', '');
                $this->setState('filter.language', $language);

                // List state information.
                parent::populateState('c.lft', 'asc');
        }

	protected function getStoreId($id = '')
        {
                // Compile the store id.
                $id     .= ':'.$this->getState('filter.search');
                $id     .= ':'.$this->getState('filter.published');
		$id     .= ':'.$this->getState('filter.language');

                return parent::getStoreId($id);
        }

	function getListQuery($resolveFKs = true)
        {
                // Create a new query object.
                $db = $this->getDbo();
                $query = $db->getQuery(true);

                // Select the required fields from the table.
                $query->select(
                        $this->getState(
                                'list.select',
                                'c.id, c.title, c.alias, c.published, c.access, c.path' .
                                ', c.checked_out, c.checked_out_time' .
                                ', c.parent_id, c.level, c.lft, c.rgt '.
				', c.language, c.created_user_id'
                        )
                );
		$query->select('COUNT(b.id) AS numitems');
                $query->from('#__abcategories AS c');

		$query->select('l.title AS language_title');
                $query->join('LEFT', '`#__languages` AS l ON l.lang_code = c.language');

		$query->select('u.name AS owner');
                $query->join('LEFT', '#__users AS u ON u.id = c.checked_out');

		$query->select('ag.title AS access_level');
                $query->join('LEFT', '#__viewlevels AS ag ON ag.id = c.access');

		$query->select('COUNT(b.id) AS numero');
                $query->join('LEFT', '#__abbook as b ON b.catid=c.id');

		// Filter by extension
                $query->where("c.extension = 'com_abook'");

		$published = $this->getState('filter.published');
                if (is_numeric($published)) {
                        $query->where('c.published = ' . (int) $published);
                } else if ($published === '') {
                        $query->where('(c.published IN (0, 1))');
                }

		// Filter by access level.
                if ($access = $this->getState('filter.access')) {
                        $query->where('c.access = ' . (int) $access);
                }

		$search = $this->getState('filter.search');
                if (!empty($search)) {
                        $search = $db->Quote('%'.$db->getEscaped($search, true).'%');
                        $query->where('(c.title LIKE '.$search.')');
                }

		// Filter on the language.
                if ($language = $this->getState('filter.language')) {
                        $query->where('c.language = '.$db->quote($language));
                }

		$query->order($db->getEscaped($this->getState('list.ordering', 'c.title')).' '.$db->getEscaped($this->getState('list.direction', 'ASC')));

		$query->group('c.id');
		
		return $query;
        }

}
